function [assign, bounds, cost, opt_k] = bin_segments(V,max_k)
%% Data binarization, compute at most k segments using Bellman's method

[T,I,J] = unique(V);
occurences = hist(J, max(J))';

n = length(T);
cumocc = [0; cumsum(occurences)];
cs = [0; cumsum(T.*occurences)];
css = [0; cumsum(T.^2.*occurences)];
sigma = zeros(length(T));

for i=1:length(T)
    for j=i+1:length(T)
        sigma(i,j) = (css(j+1) - css(i)) - (1/(cumocc(j+1)-cumocc(i)))*(cs(j+1)-cs(i)).^2;
    end
end

E = zeros(max_k,n);
E(1,:) = sigma(1,:);
B = zeros(max_k,n);

for k=2:max_k
    for cutp=k:n
        [E(k,cutp),B(k,cutp)]=min(E(k-1,[1:cutp-1])+sigma([2:cutp],cutp)');
    end
end

[cost,opt_k] = min(E(:,n));
cuts = [B(opt_k,n) n];
for j = opt_k-1:-1:2
    tmp = B(j,cuts(1));
    cuts = [tmp cuts];
end
cuts = unique([0 cuts]);
buckets = zeros(1,n);
bounds = [];
for i=1:length(cuts)-1
    buckets(cuts(i)+1:cuts(i+1)) = i;
    bounds = [bounds; T(cuts(i)+1)];
end

bounds = full(bounds(2:end));

assign = buckets(J');
if size(assign,1) > 1
    size(assign)
end


##
##    Feb, 2011. EG. ReReMi 1.0
##
