#!/bin/bash
# Generate randomized data from a given original pair of data matrix using different methods
# Possible methods are: 
# SWAP: swap randomize method Ojala et al. (for DBLP)
# SWAPDISC: swap randomize method Ojala et al. with discretization of the right hand side matrix (for Bio with climate on the right)
# PERM: random permutation of values (for Bio)
# PERMSSYM: random permutation of values, conserving symmetry of he right hand side matrix (for DBLP with coauthos on the right~/redescriptors/scripts/)

BASE_REP=../

SUFF_ORG=_0
SUFF_SWAP=_swap
SUFF_SWAPDISC=_swapdisc
SUFF_PERM=_perm
SUFF_PERMSSYM=_permssym

MAT_PATH=${BASE_REP}scripts/
METHOD_PATH=${BASE_REP}scripts/
MATLAB_BIN=/opt/matlab/bin/matlab

#####################################################################
### CHANGING PARAMETERS :

# ## DBLP
# ####################
# ORG_REP=${BASE_REP}dblp/
# DATA_REP=${BASE_REP}randomized/dblp/data/
# FILE_L=conference_picked
# FILE_R=coauthor_picked
# EXT_L=.datbool
# EXT_R=.datbool

# FIRST_COPY_SWAP=1
# LAST_COPY_SWAP=10
# FIRST_COPY_SWAPDISC=1
# LAST_COPY_SWAPDISC=0
# FIRST_COPY_PERM=1
# LAST_COPY_PERM=0
# FIRST_COPY_PERMSSYM=1
# LAST_COPY_PERMSSYM=10

# ## BIO
# ####################
ORG_REP=${BASE_REP}bio/
DATA_REP=${BASE_REP}randomized/bio/data/
FILE_L=mammals_small
FILE_R=worldclim_tp_small

EXT_L=.datbool
EXT_R=.densenum

FIRST_COPY_SWAP=1
LAST_COPY_SWAP=0
FIRST_COPY_SWAPDISC=1
LAST_COPY_SWAPDISC=10
FIRST_COPY_PERM=1
LAST_COPY_PERM=10
FIRST_COPY_PERMSSYM=1
LAST_COPY_PERMSSYM=0
#####################################################################

mkdir -p ${DATA_REP}

if [ ! -e ${DATA_REP}${FILE_L}${SUFF_ORG}${EXT_L} ]; then
    echo "Original as copy 0"
    cp ${ORG_REP}${FILE_L}${EXT_L} ${DATA_REP}${FILE_L}${SUFF_ORG}${EXT_L}
    cp ${ORG_REP}${FILE_R}${EXT_R} ${DATA_REP}${FILE_R}${SUFF_ORG}${EXT_R}
fi

####### SCRIPT MATLAB RANDOM SWAP NO DISCRETIZE
SCRIPT_MATLAB_SWAP="
path(path,'${MAT_PATH}');
%path(path,'${METHOD_PATH}');
cd '${METHOD_PATH}';
XL = load_matrix('${DATA_REP}${FILE_L}${SUFF_ORG}', '${EXT_L}');
XR = load_matrix('${DATA_REP}${FILE_R}${SUFF_ORG}', '${EXT_R}');

for i = ${FIRST_COPY_SWAP}:${LAST_COPY_SWAP}
    XL_r = swap(XL);
    XR_r = swap(XR);
    save_matrix(XL_r, ['${DATA_REP}${FILE_L}${SUFF_SWAP}' num2str(i)], '${EXT_L}');
    save_matrix(XR_r, ['${DATA_REP}${FILE_R}${SUFF_SWAP}' num2str(i)], '${EXT_R}');
end
" 

####### SCRIPT MATLAB RANDOM SWAP DISCRETIZE RIGHT
SCRIPT_MATLAB_SWAPDISC="
path(path,'${MAT_PATH}');
%path(path,'${METHOD_PATH}');
cd '${METHOD_PATH}';
XL = load_matrix('${DATA_REP}${FILE_L}${SUFF_ORG}', '${EXT_L}');
XR = load_matrix('${DATA_REP}${FILE_R}${SUFF_ORG}', '${EXT_R}');

SR = scale_data(XR);
DR = discretize(SR);

for i = ${FIRST_COPY_SWAPDISC}:${LAST_COPY_SWAPDISC}
    XL_r = swap(XL);
    DR_r = swap(DR);
    SR_r = undiscretize(DR_r, SR);
    XR_r = unscale_data(SR_r, XR);
    save_matrix(XL_r, ['${DATA_REP}${FILE_L}${SUFF_SWAPDISC}' num2str(i)], '${EXT_L}');
    save_matrix(XR_r, ['${DATA_REP}${FILE_R}${SUFF_SWAPDISC}' num2str(i)], '${EXT_R}');
end
" 

####### SCRIPT MATLAB RANDOM PERMUTE
SCRIPT_MATLAB_PERM="
path(path,'${MAT_PATH}');
path(path,'${METHOD_PATH}');
XL = load_matrix('${DATA_REP}${FILE_L}${SUFF_ORG}', '${EXT_L}');
XR = load_matrix('${DATA_REP}${FILE_R}${SUFF_ORG}', '${EXT_R}');

for i = ${FIRST_COPY_PERM}:${LAST_COPY_PERM}
    XL_r = permute_mat(XL);
    XR_r = permute_mat(XR);
    save_matrix(XL_r, ['${DATA_REP}${FILE_L}${SUFF_PERM}' num2str(i)], '${EXT_L}');
    save_matrix(XR_r, ['${DATA_REP}${FILE_R}${SUFF_PERM}' num2str(i)], '${EXT_R}');
end
"

####### SCRIPT MATLAB RANDOM PERMUTE RIGHT SPARSE SYMMETRIC
SCRIPT_MATLAB_PERMSSYM="
path(path,'${MAT_PATH}');
path(path,'${METHOD_PATH}');
XL = load_matrix('${DATA_REP}${FILE_L}${SUFF_ORG}', '${EXT_L}');
XR = load_matrix('${DATA_REP}${FILE_R}${SUFF_ORG}', '${EXT_R}');

for i = ${FIRST_COPY_PERMSSYM}:${LAST_COPY_PERMSSYM}
    XL_r = permute_mat(XL);
    XR_r = permute_ssym(XR);
    save_matrix(XL_r, ['${DATA_REP}${FILE_L}${SUFF_PERMSSYM}' num2str(i)], '${EXT_L}');
    save_matrix(XR_r, ['${DATA_REP}${FILE_R}${SUFF_PERMSSYM}' num2str(i)], '${EXT_R}');
end
"
 
if [ "$LAST_COPY_SWAP" -ge "$FIRST_COPY_SWAP" ]; then
    echo "Generating ${FIRST_COPY_SWAP}...${LAST_COPY_SWAP} swap randomized copies..."
    echo "${SCRIPT_MATLAB_SWAP}" | $MATLAB_BIN  -nosplash > /dev/null
fi
if [ "$LAST_COPY_SWAPDISC" -ge "$FIRST_COPY_SWAPDISC" ]; then
    echo "Generating ${FIRST_COPY_SWAPDISC}...${LAST_COPY_SWAPDISC} swap discretize randomized copies..."
    echo "${SCRIPT_MATLAB_SWAPDISC}" | $MATLAB_BIN  -nosplash > /dev/null
fi
if [ "$LAST_COPY_PERM" -ge "$FIRST_COPY_PERM" ]; then
    echo "Generating ${FIRST_COPY_PERM}...${LAST_COPY_PERM}  perm randomized copies..."
    echo "${SCRIPT_MATLAB_PERM}" | $MATLAB_BIN  -nosplash > /dev/null
fi
if [ "$LAST_COPY_PERMSSYM" -ge "$FIRST_COPY_PERMSSYM" ]; then
    echo "Generating ${FIRST_COPY_PERMSSYM}...${LAST_COPY_PERMSSYM}  perm sparse sym randomized copies..."
    echo "${SCRIPT_MATLAB_PERMSSYM}" | $MATLAB_BIN  -nosplash > /dev/null
fi

##
##    Feb, 2011. EG. ReReMi 1.0
##
